import { Component } from '@builder.io/qwik';
import type { QRL } from '@builder.io/qwik';
import type { ValueOrPromise } from '@builder.io/qwik';

export declare type DisplayNameFn = {
    /**
     * Return the translation of language, region, script or currency display names
     * @param code ISO code of language, region, script or currency
     * @param options Intl DisplayNamesOptions object
     * @param lang Optional language if different from the current one
     * @returns The translated code
     */
    (code: string, options: Intl.DisplayNamesOptions, lang?: string): string;
};

export declare interface DomainBasedRoutingOption {
    /**
     * Always use the lang prefix in domain-based routing, or as needed
     */
    prefix: 'always' | 'as-needed';
}

/**
 * Extract lang from domain
 */
export declare const extractFromDomain: (route: URL, domains: SpeakLocale[] | RewriteRouteOption[]) => string | undefined;

/**
 * Extract lang from url
 */
export declare const extractFromUrl: (route: URL) => string;

export declare type FormatDateFn = {
    /**
     * Format a date
     * @param value A date, a number (milliseconds since UTC epoch) or a string
     * @param options Intl DateTimeFormatOptions object
     * @param lang Optional language if different from the current one
     * @param timeZone Optional time zone if different from the current one
     * @returns The formatted date
     */
    (value: Date | number | string, options?: Intl.DateTimeFormatOptions, lang?: string, timeZone?: string): string;
};

export declare type FormatNumberFn = {
    /**
     * Format a number
     * @param value A number or a string
     * @param options Intl NumberFormatOptions object
     * @param lang Optional language if different from the current one
     * @param currency Optional currency if different from the current one
     * @returns The formatted number
     */
    (value: number | string, options?: Intl.NumberFormatOptions, lang?: string, currency?: string): string;
};

export declare const inlinePlural: () => InlinePluralFn;

export declare type InlinePluralFn = {
    /**
     * Get the plural by a number.
     * The value is passed as a parameter to the translate function
     * @param value A number or a string
     * @param key Optional key
     * @param params Optional parameters contained in the values
     * @param options Intl PluralRulesOptions object
     * @param lang Optional language if different from the current one
     * @returns The translation for the plural
     */
    (value: number | string, key?: string, params?: Record<string, any>, options?: Intl.PluralRulesOptions, lang?: string): string;
};

export declare const inlineTranslate: () => InlineTranslateFn;

export declare type InlineTranslateFn = {
    /**
     * Translate a key.
     * The syntax of the string is 'key@@[default value]'
     * @param key The key to translate
     * @param params Optional parameters contained in the value
     * @param lang Optional language if different from the current one
     * @returns The translation or the key if not found
     */
    (key: string, params?: Record<string, any>, lang?: string): string;
    <T>(key: string, params?: Record<string, any>, lang?: string): T;
    /**
     * Translate an array of keys.
     * The syntax of the strings is 'key@@[default value]'
     * @param keys The array of keys to translate
     * @param params Optional parameters contained in the values
     * @param lang Optional language if different from the current one
     * @returns The translations or the keys if not found
     */
    (keys: string[], params?: Record<string, any>, lang?: string): string[];
    <T>(keys: string[], params?: Record<string, any>, lang?: string): T[];
};

/**
 * Must contain the logic to load translation data
 */
export declare type LoadTranslationFn = QRL<(lang: string, asset: string) => ValueOrPromise<Translation | null>> | ServerQRL<(lang: string, asset: string) => ValueOrPromise<Translation | null>>;

export declare const localizePath: () => LocalizePathFn;

export declare type LocalizePathFn = {
    /**
     * Localize a path with the language
     * @param pathname The path to localize
     * @param lang Optional language if different from the default one
     * @returns The localized path
     */
    (pathname: string, lang?: string): string;
    /**
     * Localize an url with the language
     * @param url The url to localize
     * @param lang Optional language if different from the default one
     * @returns The localized url
     */
    (url: URL, lang?: string): string;
    /**
     * Localize an array of paths with the language
     * @param pathnames The array of paths to localize
     * @param lang Optional language if different from the default one
     * @returns The localized paths
     */
    (pathnames: string[], lang?: string): string[];
};

export declare interface QwikSpeakMockProps extends Omit<QwikSpeakProps, 'currency' | 'timeZone'> {
    /**
     * Optional locale to use
     */
    locale?: SpeakLocale;
}

/**
 * Create and provide the Speak context to test enviroments
 */
export declare const QwikSpeakMockProvider: Component<QwikSpeakMockProps>;

export declare interface QwikSpeakProps {
    /**
     * Speak config
     */
    config: SpeakConfig;
    /**
     * Optional functions to use
     */
    translationFn?: TranslationFn;
    /**
     * Optional additional languages to preload data for (multilingual)
     */
    langs?: string[];
    /**
     * Optional currency if different from the current one
     */
    currency?: string;
    /**
     * Optional time zone if different from the current one
     */
    timeZone?: string;
}

export declare type RelativeTimeFn = {
    /**
     * Format a relative time
     * @param value A number or a string
     * @param unit Intl RelativeTimeFormatUnit string
     * @param options Intl RelativeTimeFormatOptions object
     * @param lang Optional language if different from the current one
     */
    (value: number | string, unit: Intl.RelativeTimeFormatUnit, options?: Intl.RelativeTimeFormatOptions, lang?: string): string;
};

export declare interface RewriteRouteOption {
    /**
     * Optional prefix
     */
    prefix?: string;
    /**
     * Translated segments.
     * Key value pairs: folder name - translated value
     */
    paths: Record<string, string>;
    /**
     * In domain-based routing, set the default domain for the prefix
     */
    domain?: string;
    /**
     * In domain-based routing, set another domain for the prefix
     */
    withDomain?: string;
}

export declare type ServerQRL<T extends {
    (this: any, ...args: any[]): any;
}> = QRL<((abort: AbortSignal, ...args: Parameters<T>) => ReturnType<T>) | ((...args: Parameters<T>) => ReturnType<T>)>;

export declare interface SpeakConfig {
    /**
     * The default locale to use as fallback
     */
    defaultLocale: SpeakLocale;
    /**
     * List of locales supported by the app
     */
    supportedLocales: SpeakLocale[];
    /**
     * Translation file names.
     * Each asset is passed to the loadTranslation$ function to obtain data according to the language
     */
    assets?: string[];
    /**
     * Assets available at runtime
     */
    runtimeAssets?: string[];
    /**
     * Separator of nested keys. Default is '.'
     */
    keySeparator?: string;
    /**
     * Key-value separator. Default is '@@'
     */
    keyValueSeparator?: string;
    /**
     * Rewrite routes as specified in Vite config for qwikCity
     */
    rewriteRoutes?: RewriteRouteOption[];
    /**
     * Domain-based routing options
     */
    domainBasedRouting?: DomainBasedRoutingOption;
}

export declare interface SpeakLocale {
    /**
     * language[-script][-region]
     * Where:
     * - language: ISO 639 two-letter or three-letter code
     * - script: ISO 15924 four-letter script code
     * - region: ISO 3166 two-letter, uppercase code
     */
    lang: string;
    /**
     * Language with Intl extensions
     * language[-script][-region][-extensions]
     */
    extension?: string;
    /**
     * ISO 4217 three-letter code
     */
    currency?: string;
    /**
     * Time zone name from the IANA time zone database
     */
    timeZone?: string;
    /**
     * Key value pairs of unit identifiers
     */
    units?: Record<string, string>;
    /**
     * Text direction
     */
    dir?: 'ltr' | 'rtl' | 'auto';
    /**
     * In domain-based routing, set the default domain for the locale
     */
    domain?: string;
    /**
     * In domain-based routing, set another domain for the locale
     */
    withDomain?: string;
}

export declare interface SpeakProps {
    /**
     * Assets to load
     */
    assets?: string[];
    /**
     * Assets to load available at runtime
     */
    runtimeAssets?: string[];
    /**
     * Optional additional languages to preload data for (multilingual)
     */
    langs?: string[];
}

export declare interface SpeakState {
    /**
     * Current locale
     */
    locale: SpeakLocale;
    /**
     * Translation data
     */
    translation: Translation;
    /***
     * Speak configuration
     */
    config: SpeakConfig;
    /**
     * Functions to use
     */
    translationFn: TranslationFn;
}

/**
 * In prod, handle the needed prefixes in domain-based routing
 */
export declare function toPrefixAsNeeded(rewriteRoutes: RewriteRouteOption[]): RewriteRouteOption[];

export declare const translatePath: () => TranslatePathFn;

export declare type TranslatePathFn = {
    /**
     * Translate a path
     * @param pathname The path to translate
     * @param lang Optional language if different from the default one
     * @returns The translation or the path if not found
     */
    (pathname: string, lang?: string): string;
    /**
     * Translate an url
     * @param url The url to translate
     * @param lang Optional language if different from the default one
     * @returns The translation or the url if not found
     */
    (url: URL, lang?: string): string;
    /**
     * Translate an array of paths
     * @param pathname The array of paths to translate
     * @param lang Optional language if different from the default one
     * @returns The translations or the paths if not found
     */
    (pathnames: string[], lang?: string): string[];
};

/**
 * Translation data
 */
export declare type Translation = {
    [key: string]: any;
};

export declare interface TranslationFn {
    /**
     * Function to load translation data
     */
    loadTranslation$: LoadTranslationFn;
}

export declare const useDisplayName: () => DisplayNameFn;

export declare const useFormatDate: () => FormatDateFn;

export declare const useFormatNumber: () => FormatNumberFn;

/**
 * Create and provide the Speak context.
 * Translations will be available in the whole app
 */
export declare const useQwikSpeak: (props: QwikSpeakProps) => void;

export declare const useRelativeTime: () => RelativeTimeFn;

/**
 * Add translation data to the context.
 * Translations will only be available in child components
 */
export declare const useSpeak: (props: SpeakProps) => void;

export declare const useSpeakConfig: () => SpeakConfig;

export declare const useSpeakContext: () => SpeakState;

export declare const useSpeakLocale: () => SpeakLocale;

/**
 * Validate language[-script][-region]
 * - `language` ISO 639 two-letter or three-letter code
 * - `script` ISO 15924 four-letter script code
 * - `region` ISO 3166 two-letter, uppercase code
 */
export declare const validateLocale: (lang: string) => boolean;

export { }
