"use strict";
/**
 * @license
 * Qwik Speak
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/robisim74/qwik-speak/blob/main/LICENSE
 */
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const build = require("@builder.io/qwik/build");
const qwik = require("@builder.io/qwik");
const SpeakContext = qwik.createContextId("qwik-speak");
const _speakServerContext = {
  translation: {},
  config: {}
};
const _speakClientContext = {
  translation: {},
  config: {},
  locale: {}
};
const getSpeakContext = () => {
  if (build.isServer)
    return _speakServerContext;
  else
    return _speakClientContext;
};
let getLang = () => "";
const setGetLangFn = (fn) => {
  getLang = () => fn();
};
const logWarn = (message) => {
  console.warn("\x1B[33mQwik Speak warn\x1B[0m %s", message);
};
const logDebug = (message) => {
  console.debug("\x1B[36mQwik Speak\x1B[0m %s", message);
};
const cache = {};
const memoize = (fn) => {
  return (...args) => {
    const stringArgs = JSON.stringify(args);
    return stringArgs in cache ? cache[stringArgs] : cache[stringArgs] = fn(...args);
  };
};
const loadTranslations = async (ctx, assets, runtimeAssets, langs) => {
  if (build.isServer || runtimeAssets) {
    const { locale, translation, translationFn, config } = ctx;
    const { translation: _translation } = getSpeakContext();
    if (build.isDev) {
      const conflictingAsset = assets?.find((asset) => runtimeAssets?.includes(asset)) || assets?.find((asset) => config.runtimeAssets?.includes(asset)) || runtimeAssets?.find((asset) => config.assets?.includes(asset));
      if (conflictingAsset)
        logWarn(`Conflict between assets and runtimeAssets '${conflictingAsset}'`);
    }
    let resolvedAssets;
    if (build.isServer)
      resolvedAssets = [
        ...assets ?? [],
        ...runtimeAssets ?? []
      ];
    else
      resolvedAssets = [
        ...runtimeAssets ?? []
      ];
    if (resolvedAssets.length === 0)
      return;
    const resolvedLangs = new Set(langs || []);
    resolvedLangs.add(locale.lang);
    for (const lang of resolvedLangs) {
      let tasks;
      if (!build.isDev) {
        const memoized = memoize(translationFn.loadTranslation$);
        tasks = resolvedAssets.map((asset) => memoized(lang, asset));
      } else
        tasks = resolvedAssets.map((asset) => translationFn.loadTranslation$(lang, asset));
      const sources = await Promise.all(tasks);
      const assetSources = sources.map((source, i) => ({
        asset: resolvedAssets[i],
        source
      }));
      if (!(lang in _translation))
        Object.assign(_translation, {
          [lang]: {}
        });
      for (const data of assetSources)
        if (data?.source) {
          if (build.isServer) {
            if (assets?.includes(data.asset))
              Object.assign(_translation[lang], data.source);
            else {
              Object.assign(_translation[lang], data.source);
              Object.assign(translation[lang], data.source);
            }
          } else
            Object.assign(_translation[lang], data.source);
        }
    }
  }
};
const getValue = (key, data, params, keySeparator = ".", keyValueSeparator = "@@") => {
  let defaultValue = void 0;
  [key, defaultValue] = separateKeyValue(key, keyValueSeparator);
  const value = key.split(keySeparator).reduce((acc, cur) => acc && acc[cur] !== void 0 ? acc[cur] : void 0, data);
  if (value) {
    if (typeof value === "string")
      return params ? transpileParams(value, params) : value;
    if (typeof value === "object")
      return params ? JSON.parse(transpileParams(JSON.stringify(value), params)) : value;
  }
  if (defaultValue) {
    if (!/^[[{].*[\]}]$/.test(defaultValue) || /^{{/.test(defaultValue))
      return params ? transpileParams(defaultValue, params) : defaultValue;
    return params ? JSON.parse(transpileParams(defaultValue, params)) : JSON.parse(defaultValue);
  }
  return key;
};
const separateKeyValue = (key, keyValueSeparator = "@@") => {
  return key.split(keyValueSeparator);
};
const transpileParams = (value, params) => {
  const replacedValue = value.replace(/{{\s?([^{}\s]*)\s?}}/g, (substring, parsedKey) => {
    const replacer = params[parsedKey];
    return replacer !== void 0 ? replacer : substring;
  });
  return replacedValue;
};
const inlineTranslate = () => {
  const currentLang = getLang();
  const translate = (keys, params, lang) => {
    const { translation, config } = getSpeakContext();
    lang ?? (lang = currentLang);
    if (Array.isArray(keys))
      return keys.map((k) => getValue(k, translation[lang], params, config.keySeparator, config.keyValueSeparator));
    return getValue(keys, translation[lang], params, config.keySeparator, config.keyValueSeparator);
  };
  return translate;
};
const inlinePlural = () => {
  const currentLang = getLang();
  const plural = (value, key, params, options, lang) => {
    const { translation, config } = getSpeakContext();
    lang ?? (lang = currentLang);
    value = +value;
    const rule = new Intl.PluralRules(lang, options).select(value);
    let defaultValues = void 0;
    if (key) {
      [key, defaultValues] = separateKeyValue(key, config.keyValueSeparator);
      if (!defaultValues && /^{.*}$/.test(key)) {
        defaultValues = key;
        key = void 0;
      }
    }
    key = key ? `${key}${config.keySeparator}${rule}` : rule;
    const defaultValue = defaultValues ? JSON.parse(defaultValues)[rule] : void 0;
    if (defaultValue)
      key = `${key}${config.keyValueSeparator}${defaultValue}`;
    return getValue(key, translation[lang], {
      value,
      ...params
    }, config.keySeparator, config.keyValueSeparator);
  };
  return plural;
};
const useQwikSpeak = (props) => {
  const lang = qwik.getLocale("");
  let resolvedLocale = props.config.supportedLocales.find((value) => value.lang === lang);
  if (!resolvedLocale) {
    resolvedLocale = props.config.defaultLocale;
    if (build.isDev)
      logWarn(`Locale not resolved. Fallback to default locale: ${props.config.defaultLocale.lang}`);
  } else if (build.isDev)
    logDebug(`Resolved locale: ${resolvedLocale.lang}`);
  if (props.currency)
    resolvedLocale.currency = props.currency;
  if (props.timeZone)
    resolvedLocale.timeZone = props.timeZone;
  const resolvedConfig = {
    rewriteRoutes: props.config.rewriteRoutes,
    defaultLocale: props.config.defaultLocale,
    supportedLocales: props.config.supportedLocales,
    assets: props.config.assets,
    runtimeAssets: props.config.runtimeAssets,
    keySeparator: props.config.keySeparator || ".",
    keyValueSeparator: props.config.keyValueSeparator || "@@",
    domainBasedRouting: props.config.domainBasedRouting
  };
  const resolvedTranslationFn = {
    loadTranslation$: props.translationFn?.loadTranslation$ ?? /* @__PURE__ */ qwik.inlinedQrl(() => null, "useQwikSpeak_resolvedTranslationFn_T9Fy9UxepgA")
  };
  const state = {
    locale: Object.assign({}, resolvedLocale),
    translation: Object.fromEntries(resolvedConfig.supportedLocales.map((value) => [
      value.lang,
      {}
    ])),
    config: Object.assign({}, resolvedConfig),
    translationFn: resolvedTranslationFn
  };
  const { config } = state;
  const { config: _config } = getSpeakContext();
  Object.assign(_config, config);
  setGetLangFn(() => qwik.getLocale(config.defaultLocale.lang));
  qwik.useContextProvider(SpeakContext, state);
  qwik.useTaskQrl(/* @__PURE__ */ qwik.inlinedQrl(async () => {
    const [config2, props2, state2] = qwik.useLexicalScope();
    if (build.isServer)
      await loadTranslations(state2, config2.assets, config2.runtimeAssets, props2.langs);
  }, "useQwikSpeak_useTask_0fwIeNJbECQ", [
    config,
    props,
    state
  ]));
  const resumeContext$ = /* @__PURE__ */ qwik.inlinedQrl(() => {
    const [state2] = qwik.useLexicalScope();
    const { locale, translation, config: config2 } = state2;
    const _speakContext = getSpeakContext();
    const { locale: _locale, translation: _translation, config: _config2 } = _speakContext;
    Object.assign(_locale, locale);
    Object.assign(_translation, translation);
    Object.assign(_config2, config2);
    setGetLangFn(() => locale.lang);
    if (build.isDev)
      console.debug("%cQwik Speak Inline", "background: #0c75d2; color: white; padding: 2px 3px; border-radius: 2px; font-size: 0.8em;", "Client context", _speakContext);
    if (build.isDev) {
      console.debug("%cQwik Speak Inline", "background: #0c75d2; color: white; padding: 2px 3px; border-radius: 2px; font-size: 0.8em;", "Ready");
      if (void 0)
        (void 0).send("qwik-speak:lang", {
          msg: locale.lang
        });
    }
  }, "useQwikSpeak_resumeContext_6bH92HveK8o", [
    state
  ]);
  qwik.useOnWindow("load", resumeContext$);
};
const QwikSpeakMockProvider = /* @__PURE__ */ qwik.componentQrl(/* @__PURE__ */ qwik.inlinedQrl((props) => {
  const lang = props.locale?.lang;
  let resolvedLocale = props.config.supportedLocales.find((value) => value.lang === lang);
  if (!resolvedLocale)
    resolvedLocale = props.config.defaultLocale;
  const resolvedConfig = {
    rewriteRoutes: props.config.rewriteRoutes,
    defaultLocale: props.config.defaultLocale,
    supportedLocales: props.config.supportedLocales,
    assets: props.config.assets,
    runtimeAssets: props.config.runtimeAssets,
    keySeparator: props.config.keySeparator || ".",
    keyValueSeparator: props.config.keyValueSeparator || "@@",
    domainBasedRouting: props.config.domainBasedRouting
  };
  const resolvedTranslationFn = {
    loadTranslation$: props.translationFn?.loadTranslation$ ?? /* @__PURE__ */ qwik.inlinedQrl(() => null, "QwikSpeakMockProvider_component_resolvedTranslationFn_aOSQ00KYdkc")
  };
  const state = {
    locale: Object.assign({}, resolvedLocale),
    translation: Object.fromEntries(props.config.supportedLocales.map((value) => [
      value.lang,
      {}
    ])),
    config: Object.assign({}, resolvedConfig),
    translationFn: resolvedTranslationFn
  };
  const { locale, config } = state;
  const { config: _config } = getSpeakContext();
  Object.assign(_config, config);
  setGetLangFn(() => locale.lang);
  qwik.useContextProvider(SpeakContext, state);
  qwik.useTaskQrl(/* @__PURE__ */ qwik.inlinedQrl(async () => {
    const [config2, props2, state2] = qwik.useLexicalScope();
    await loadTranslations(state2, config2.assets, config2.runtimeAssets, props2.langs);
  }, "QwikSpeakMockProvider_component_useTask_0eOz05v4yM0", [
    config,
    props,
    state
  ]));
  return /* @__PURE__ */ qwik._jsxC(qwik.Slot, null, 3, "6l_0");
}, "QwikSpeakMockProvider_component_zQJA0JbOhj0"));
const useSpeakContext = () => qwik.useContext(SpeakContext);
const useSpeakLocale = () => qwik.useContext(SpeakContext).locale;
const useSpeakConfig = () => qwik.useContext(SpeakContext).config;
const useSpeak = (props) => {
  const ctx = useSpeakContext();
  const { config } = ctx;
  qwik.useTaskQrl(/* @__PURE__ */ qwik.inlinedQrl(async () => {
    const [config2, ctx2, props2] = qwik.useLexicalScope();
    if (build.isDev) {
      if (!props2.assets && !props2.runtimeAssets)
        logWarn("useSpeak: no assets provided");
      const duplicateAsset = config2.assets?.find((asset) => props2.assets?.includes(asset));
      if (duplicateAsset)
        logWarn(`useSpeak: duplicate assets '${duplicateAsset}'`);
      const duplicateRuntimeAsset = config2.runtimeAssets?.find((asset) => props2.runtimeAssets?.includes(asset));
      if (duplicateRuntimeAsset)
        logWarn(`useSpeak: duplicate runtimeAssets '${duplicateRuntimeAsset}'`);
    }
    await loadTranslations(ctx2, props2.assets, props2.runtimeAssets, props2.langs);
    if (build.isDev && build.isBrowser) {
      const _speakContext = getSpeakContext();
      console.debug("%cQwik Speak Inline", "background: #0c75d2; color: white; padding: 2px 3px; border-radius: 2px; font-size: 0.8em;", "Client context", _speakContext);
    }
  }, "useSpeak_useTask_3ViHzm0oQjw", [
    config,
    ctx,
    props
  ]));
};
const useFormatNumber = () => {
  const locale = useSpeakLocale();
  const formatNumber = (value, options, lang, currency) => {
    lang ?? (lang = locale.extension ?? locale.lang);
    currency = currency ?? locale.currency;
    value = +value;
    options = {
      ...options
    };
    if (currency)
      options.currency = currency;
    return new Intl.NumberFormat(lang, options).format(value);
  };
  return formatNumber;
};
const useFormatDate = () => {
  const locale = useSpeakLocale();
  const formateDate = (value, options, lang, timeZone) => {
    lang ?? (lang = locale.extension ?? locale.lang);
    timeZone = timeZone ?? locale.timeZone;
    value = new Date(value);
    options = {
      ...options
    };
    if (timeZone)
      options.timeZone = timeZone;
    return new Intl.DateTimeFormat(lang, options).format(value);
  };
  return formateDate;
};
const useRelativeTime = () => {
  const locale = useSpeakLocale();
  const relativeTime = (value, unit, options, lang) => {
    lang ?? (lang = locale.extension ?? locale.lang);
    value = +value;
    return new Intl.RelativeTimeFormat(lang, options).format(value, unit);
  };
  return relativeTime;
};
const useDisplayName = () => {
  const locale = useSpeakLocale();
  const displayName = (code, options, lang) => {
    lang ?? (lang = locale.extension ?? locale.lang);
    return new Intl.DisplayNames(lang, options).of(code) || code;
  };
  return displayName;
};
const localizePath = () => {
  const { config } = getSpeakContext();
  const currentLang = getLang();
  const getRegEpx = (lang) => new RegExp(`(/${lang}/)|(/${lang}$)|(/(${lang})(?=\\?))`);
  const replace = (pathname, lang) => {
    const langParam = config.supportedLocales.find((locale) => getRegEpx(locale.lang)?.test(pathname))?.lang;
    if (langParam) {
      if (lang !== config.defaultLocale.lang)
        pathname = pathname.replace(langParam, lang);
      else
        pathname = pathname.replace(getRegEpx(langParam), "/");
    } else if (lang !== config.defaultLocale.lang)
      pathname = `/${lang}${pathname}`;
    if (!build.isDev) {
      if (config.domainBasedRouting?.prefix === "as-needed" && isDefaultDomain(lang))
        pathname = pathname.replace(getRegEpx(lang), "/");
    }
    return pathname;
  };
  const localize = (route, lang) => {
    lang ?? (lang = currentLang);
    if (Array.isArray(route))
      return route.map((path) => replace(path, lang));
    if (typeof route === "string")
      return replace(route, lang);
    let url = new URL(route);
    if (config.domainBasedRouting)
      url = localizeDomain(url, lang);
    url.pathname = replace(url.pathname, lang);
    return url.toString().replace(/\/\?/, "?");
  };
  return localize;
};
const translatePath = () => {
  const { config } = getSpeakContext();
  const rewriteRoutes = config.rewriteRoutes;
  const currentLang = getLang();
  const normalizePath = (pathname) => {
    let source = rewriteRoutes?.find((rewrite) => pathname === `/${rewrite.prefix}` || pathname.startsWith(`/${rewrite.prefix}/`) || pathname.startsWith(`${rewrite.prefix}/`));
    if (source)
      pathname = pathname === `/${source.prefix}` ? "/" : pathname.startsWith("/") ? pathname.substring(`/${source.prefix}`.length) : pathname.substring(`${source.prefix}/`.length);
    if (!build.isDev) {
      if (config.domainBasedRouting?.prefix === "as-needed") {
        if (!source)
          source = rewriteRoutes?.find((rewrite) => rewrite.lang == currentLang);
      }
    }
    if (source) {
      const sourceEntries = Object.entries(source.paths).map(([from, to]) => [
        to,
        from
      ]);
      const revertedPaths = Object.fromEntries(sourceEntries);
      const splitted = pathname.split("/");
      const translated = splitted.map((part) => revertedPaths[part] ?? part);
      return translated.join("/");
    }
    return pathname;
  };
  const rewritePath = (pathname, prefix) => {
    let splitted = pathname.split("/");
    let destination = rewriteRoutes?.find((rewrite) => rewrite.prefix === prefix);
    if (prefix && destination) {
      const keys = Object.keys(destination.paths);
      const translating = splitted.some((part) => keys.includes(part));
      if (pathname === "/" || translating)
        pathname = pathname.startsWith("/") ? `/${prefix}` + pathname : `${prefix}/` + pathname;
    }
    if (!build.isDev) {
      if (config.domainBasedRouting?.prefix === "as-needed") {
        if (prefix && !destination)
          destination = rewriteRoutes?.find((rewrite) => rewrite.lang === prefix);
      }
    }
    splitted = pathname.split("/");
    const translated = splitted.map((part) => destination?.paths[part] ?? part);
    return translated.join("/");
  };
  const slashPath = (pathname, rewrote) => {
    if (pathname.endsWith("/") && !rewrote.endsWith("/"))
      return rewrote + "/";
    else if (!pathname.endsWith("/") && rewrote.endsWith("/") && rewrote !== "/")
      return rewrote.substring(0, rewrote.length - 1);
    return rewrote;
  };
  const translateOne = (pathname, lang) => {
    const normalized = normalizePath(pathname);
    const rewrote = rewritePath(normalized, lang);
    return slashPath(pathname, rewrote);
  };
  const translate = (route, lang) => {
    lang ?? (lang = currentLang);
    if (!rewriteRoutes) {
      if (build.isDev)
        logWarn(`translatePath: rewriteRoutes not found`);
      return route;
    }
    if (Array.isArray(route))
      return route.map((path) => translateOne(path, lang));
    if (typeof route === "string")
      return translateOne(route, lang);
    let url = new URL(route);
    if (config.domainBasedRouting)
      url = translateDomain(url, lang);
    url.pathname = translateOne(url.pathname, lang);
    return url.toString();
  };
  return translate;
};
const extractFromDomain = (route, domains) => {
  const hostname = !route.hostname.startsWith("localhost") ? route.hostname : route.host;
  const domain = domains.find((value) => value.domain === hostname);
  return domain && ("lang" in domain ? domain.lang : "prefix" in domain ? domain.prefix : void 0);
};
const extractFromUrl = (route) => {
  const parts = route.pathname.split("/");
  return route.pathname.startsWith("/") ? parts[1] : parts[0];
};
const validateLocale = (lang) => {
  return /^([a-z]{2,3})(-[A-Z][a-z]{3})?(-[A-Z]{2})?$/.test(lang);
};
function toPrefixAsNeeded(rewriteRoutes) {
  if (build.isDev)
    return rewriteRoutes;
  const routes = rewriteRoutes.map((rewrite) => ({
    prefix: rewrite.domain ? void 0 : rewrite.prefix,
    paths: rewrite.paths,
    lang: rewrite.prefix,
    domain: rewrite.domain,
    withDomain: rewrite.withDomain
  }));
  return routes;
}
const localizeDomain = (url, lang) => {
  const { config } = getSpeakContext();
  const locale = config.supportedLocales.find((value) => value.lang === lang);
  const domain = locale?.domain || locale?.withDomain;
  if (!domain) {
    if (build.isDev)
      logWarn(`localizeDomain: domain not found`);
  } else if (!build.isDev) {
    if (!domain.startsWith("localhost"))
      url.hostname = domain;
    else
      url.host = domain;
  }
  return url;
};
const translateDomain = (url, lang) => {
  const { config } = getSpeakContext();
  const rewriteRoutes = config.rewriteRoutes;
  const rewrite = rewriteRoutes?.find((value) => value.lang === lang || value.prefix === lang);
  const domain = rewrite?.domain || rewrite?.withDomain || // Default locale
  rewriteRoutes.find((rewrite2) => rewrite2.domain && Object.keys(rewrite2.paths).length === 0)?.domain;
  if (!domain) {
    if (build.isDev)
      logWarn(`translateDomain: domain not found`);
  } else if (!build.isDev) {
    if (!domain.startsWith("localhost"))
      url.hostname = domain;
    else
      url.host = domain;
  }
  return url;
};
const isDefaultDomain = (lang) => {
  const { config } = getSpeakContext();
  return config.supportedLocales.find((value) => value.lang === lang)?.domain !== void 0;
};
exports.QwikSpeakMockProvider = QwikSpeakMockProvider;
exports.extractFromDomain = extractFromDomain;
exports.extractFromUrl = extractFromUrl;
exports.inlinePlural = inlinePlural;
exports.inlineTranslate = inlineTranslate;
exports.localizePath = localizePath;
exports.toPrefixAsNeeded = toPrefixAsNeeded;
exports.translatePath = translatePath;
exports.useDisplayName = useDisplayName;
exports.useFormatDate = useFormatDate;
exports.useFormatNumber = useFormatNumber;
exports.useQwikSpeak = useQwikSpeak;
exports.useRelativeTime = useRelativeTime;
exports.useSpeak = useSpeak;
exports.useSpeakConfig = useSpeakConfig;
exports.useSpeakContext = useSpeakContext;
exports.useSpeakLocale = useSpeakLocale;
exports.validateLocale = validateLocale;
